/* -*- mode: c++; c-basic-offset: 4; -*- */
//
//    Build a channel list from a specified frame
#include "framecpp/FrameH.hh"
#include "framecpp/FrVect.hh"
// #include "framecpp/Dimension.hh"
#include "framecpp/FrRawData.hh"
#include "framecpp/FrAdcData.hh"
#include "framecpp/FrProcData.hh"
#include "framecpp/Common/IOStream.hh"
#include "framecpp/IFrameStream.hh"
#include "ldastoolsal/fstream.hh"

#include "data_type.hh"
#include "iSMbuf.hh"
#include <string>
#include <iostream>
#include <iomanip>
#include <map>

//======================================  Define pointer types
#define FCPP_SHARED_PTRS 1
#include "ldastoolsal/SharedPtr.hh"
typedef LDASTools::AL::SharedPtr<FrameCPP::FrameH>     frame_pointer;
typedef LDASTools::AL::SharedPtr<FrameCPP::FrRawData>  frrawdata_pointer;
typedef LDASTools::AL::SharedPtr<FrameCPP::FrAdcData>  fradcdata_pointer;
typedef LDASTools::AL::SharedPtr<FrameCPP::FrVect>     frvect_pointer;

#define PARTITION_TAG "/online/"

using namespace std;
using FrameCPP::FrameH;
using FrameCPP::FrAdcData;
using FrameCPP::FrRawData;
using FrameCPP::FrProcData;
using FrameCPP::FrVect;
using FrameCPP::IFrameStream;

typedef LDASTools::AL::filebuf filebuf_type;
const char*
str_data_type(frvect_pointer v) {
    sends::data_type dt;
    dt.set_frame_type(v->GetType());
    return dt.str();
}

//======================================  Frame comparison main function
int
main(int argc, const char* argv[]) {
    //----------------------------------  Parse the command line arguments
    int  verbose = 0;
    bool ratemsg = false;
    bool print_hdr = true;
    int  access_lvl = 0;
    bool syntax  = false;
    string traw = "raw";
    for (int i=1 ; i<argc-1 ; i++) {
        string argi = argv[i];
        if (argi == "-a") {
	    access_lvl = strtol(argv[++i], 0, 0);
	} else if (argi == "-h") {
	    print_hdr = false;
	} else if (argi == "-t") {
	    traw = argv[++i];
	} else if (argi == "-v") {
	    verbose++;
	} else {
	    syntax = true;
	    cerr << "Unrecognized argument: " << argi << endl;
	}
    }

    //-----------------------------------  Print syntax and bail on error/help
    if (syntax || argc < 2 || *argv[argc-1] == '-') { 
        cerr << "Command Syntax: " << endl;
	cerr << "build-channel-list"
	     << " [-v] [-a <access>] [-t <raw-type>]"
	     << " <file>" 
	     << endl;
	return 1;
    }

    //-----------------------------------  Parse the file name
    string file(argv[argc-1]);
    string::size_type inxm = file.rfind("/");
    string sourcename;
    if (inxm == string::npos) sourcename = file;
    else                      sourcename = file.substr(inxm+1);
    inxm = sourcename.find('-', 0);
    if (inxm < string::npos) inxm = sourcename.find('-', inxm+1);
    if (inxm < string::npos) sourcename = sourcename.erase(inxm);


    FrameCPP::Common::FrameBufferInterface* fbuf = 0; 

    size_t ltag = strlen(PARTITION_TAG);
    if (file.substr(0,ltag) != PARTITION_TAG) {
	FrameCPP::Common::FrameBuffer<filebuf_type>*
	    sbuf = new FrameCPP::Common::FrameBuffer<filebuf_type>(ios::in);
	if (!sbuf->open(file.c_str(), ios::in)) {
	    cerr << "Unable to open frame file: " << file << endl;
	    return 2;
	}
	fbuf = sbuf;
    }
    else {
	const char* part = file.c_str() + ltag;
	FrameCPP::Common::FrameBuffer< iSMbuf >*
	    sbuf = new  FrameCPP::Common::FrameBuffer< iSMbuf >(ios::in);
        if (!sbuf) {
            cerr << "Unable to construct FrameBuffer<iSMbuf> for partition " 
                 << part << endl;
            return 2;
        }
        if (! static_cast<iSMbuf*>(sbuf)->open(part, ios::in)) {
            cerr << "Unable to open partition " << part << endl;
            delete sbuf;
            return 2;
        }
	fbuf = sbuf;
    }
    if (print_hdr) {
	cout <<"#   Channel list generated from: " << file << endl;
    }

    IFrameStream ifstr(fbuf);
    frame_pointer frame(ifstr.ReadNextFrame());

    //----------------------------------  Build a list of FrAdcData channels
    frrawdata_pointer raw(frame->GetRawData());
    if (!raw) {
	if (verbose) cerr << "File has no raw data." << endl;
    } else {
	FrRawData::const_firstAdc_iterator adcIter=raw->RefFirstAdc().begin();
	for ( ; adcIter != raw->RefFirstAdc().end() ; adcIter++ ) {
	    fradcdata_pointer ap(*adcIter);
	    string name = ap->GetName();
	    string type = traw;
	    double rate = ap->GetSampleRate();
	    frvect_pointer    vp = ap->RefData().front();

	    //--------------------------  Test the rate field against dx
	    double rdxprod = rate * vp->GetDim(0).GetDx();
	    if (fabs(rdxprod - 1.0) > 1e-6) {
		if (!ratemsg) {
		    cerr << "Sample rate inconsistent for channel: " << name
			 << " in file: " << file << endl;
		    ratemsg = true;
		}
		rate = 1.0/vp->GetDim(0).GetDx();
	    }

	    //--------------------------  Look for trend subchannel name.
	    string::size_type inx = name.find_last_of(".");
	    if (inx != string::npos) {
		string ext = name.substr(inx);
		if (ext == ".n" || ext == ".mean" || ext == ".rms" ||
		    ext == ".min" || ext == ".max") {
		    if (rate == 1.0) {
			type = "s-trend";
		    } else {
			type = "m-trend";
		    }
		}
	    }

	    //--------------------------  Print the channel name.
	    cout << name << " " << type << " " << access_lvl << " " 
		 << str_data_type(vp) << " " << rate << " " 
		 << sourcename << endl;
	}
    }

    //----------------------------------  Build a list of FrProcData channels
    for (FrameH::const_procData_iterator i=frame->RefProcData().begin();
	 i != frame->RefProcData().end(); i++) {
	string type = "reduced";
	if (traw == "online") type = traw;
	frvect_pointer vp = (*i)->RefData().front();
	double rate = 1./vp->GetDim(0).GetDx();
	cout << (*i)->GetName() << " " << type << " " << access_lvl << " "
	     << str_data_type(vp) << " " << rate << " " 
	     << sourcename << endl;
    }
#ifndef FCPP_SHARED_PTRS
    delete frame;
#endif
    return 0;
}

#include "framecpp/Common/IOStream.icc"
 template class FrameCPP::Common::FrameBuffer< iSMbuf >;
